package table;

import java.util.HashMap;

import function.LispFunction;
import function.builtin.*;
import function.builtin.cons.*;
import function.builtin.math.*;
import function.builtin.predicate.*;
import function.builtin.special.*;

public class FunctionTable {
    
    private static FunctionTable uniqueInstance = new FunctionTable();
    
    public static LispFunction lookupFunction(String functionName) {
        return uniqueInstance.functionTable.get(functionName);
    }
    
    public static boolean isAlreadyDefined(String functionName) {
        return uniqueInstance.functionTable.containsKey(functionName);
    }
    
    public static void defineFunction(String functionName, LispFunction function) {
        uniqueInstance.functionTable.put(functionName, function);
    }
    
    private HashMap<String, LispFunction> functionTable;

    private FunctionTable() {
        this.functionTable = new HashMap<>();
        
        initializeFunctionTable();
    }

    private void initializeFunctionTable() {
        functionTable.put("*", new MULTIPLY());
        functionTable.put("+", new PLUS());
        functionTable.put("-", new MINUS());
        functionTable.put("/", new DIVIDE());
        functionTable.put("<", new LESSP());
        functionTable.put("=", new EQUALSP());
        functionTable.put(">", new GREATERP());
        functionTable.put("APPLY", new APPLY());
        functionTable.put("ATOM", new ATOM());
        functionTable.put("CAR", new CAR());
        functionTable.put("CDR", new CDR());
        functionTable.put("COND", new COND());
        functionTable.put("CONS", new CONS());
        functionTable.put("DEFUN", new DEFUN());
        functionTable.put("EQ", new EQ());
        functionTable.put("EQUAL", new EQUAL());
        functionTable.put("EVAL", new EVAL());
        functionTable.put("EXIT", new EXIT());
        functionTable.put("FIRST", new CAR());
        functionTable.put("FUNCALL", new FUNCALL());
        functionTable.put("GREATERP", new GREATERP());
        functionTable.put("LAMBDA", new LAMBDA());
        functionTable.put("LENGTH", new LENGTH());
        functionTable.put("LET", new LET());
        functionTable.put("LIST", new LIST());
        functionTable.put("LISTP", new LISTP());
        functionTable.put("LOAD", new LOAD());
        functionTable.put("NULL", new NULL());
        functionTable.put("PRINT", new PRINT());
        functionTable.put("QUOTE", new QUOTE());
        functionTable.put("REST", new CDR());
        functionTable.put("SETF", new SETF());
        functionTable.put("SYMBOL-FUNCTION", new SYMBOL_FUNCTION());
    }

}
